#include <sys/types.h>
#include <sys/wait.h>
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

char *newenv[] = { "__FE_FORKED__=yes", NULL };

int
main(int argc, char *argv[])
{
	int status;
	pid_t pid;

	/* Check if we exec'd ourselves after a fork. */
	if (getenv("__FE_FORKED__") != NULL) {
		printf("%s(%d): child after exec. Exiting...\n",
		    getprogname(), getpid());
		return (0);
	}

	printf("%s(%d): initial process. Doing fork & exec...\n",
	    getprogname(), getpid());

	pid = fork();
	switch (pid) {
	default:	/* parent */
		status = -1;
		pid = wait(&status);
		if (pid == -1)
			printf("%s(%d): wait(2) failed with error %d (%s)\n",
			    getprogname(), getpid(), errno, strerror(errno));
		else
			printf("%s(%d): child %d exited with status %d\n",
			    getprogname(), getpid(), pid, status);
		break;
	case 0:		/* child */
		printf("%s(%d): child after fork. Doing exec...\n",
		    getprogname(), getpid());
		setenv("__FE_FORKED__", "done", 1);
		execve(getprogname(), argv, newenv);
		err(2, "exec failed");
		break;
	case -1:
		err(1, "fork failed");
	}

	return (0);
}
